@@ -38,7 +38,7 @@ APPLET_NAME = 'firewall-applet'
 DATADIR = '/usr/share/' + DAEMON_NAME
 CONFIG_GLADE_NAME = CONFIG_NAME + '.glade'
 COPYRIGHT = '(C) 2010-2012 Red Hat, Inc.'
-VERSION = '0.3.11'
+VERSION = '0.3.12'
 AUTHORS = [
     "Thomas Woerner <twoerner@redhat.com>",
     "Jiri Popelka <jpopelka@redhat.com>",
@@ -56,7 +56,7 @@ LICENSE = _(
     "\n"
     "You should have received a copy of the GNU General Public License "
     "along with this program.  If not, see <http://www.gnu.org/licenses/>.")
-WEBSITE = 'https://fedorahosted.org/firewalld/'
+WEBSITE = 'http://www.firewalld.org'
 
 ETC_FIREWALLD = '/etc/firewalld'
 FIREWALLD_CONF = ETC_FIREWALLD + '/firewalld.conf'
