@@ -31,6 +31,7 @@ DBUS_INTERFACE_CONFIG_ZONE = DBUS_INTERFACE_CONFIG+".zone"
 DBUS_INTERFACE_CONFIG_SERVICE = DBUS_INTERFACE_CONFIG+".service"
 DBUS_INTERFACE_CONFIG_ICMPTYPE = DBUS_INTERFACE_CONFIG+".icmptype"
 DBUS_INTERFACE_CONFIG_POLICIES = DBUS_INTERFACE_CONFIG+".policies"
+DBUS_INTERFACE_CONFIG_DIRECT = DBUS_INTERFACE_CONFIG+".direct"
 
 DBUS_PATH = "/org/fedoraproject/FirewallD%d" % DBUS_INTERFACE_VERSION
 DBUS_PATH_CONFIG = DBUS_PATH+"/config"
@@ -38,9 +39,13 @@ DBUS_PATH_CONFIG_ICMPTYPE = DBUS_PATH+"/config/icmptype"
 DBUS_PATH_CONFIG_SERVICE = DBUS_PATH+"/config/service"
 DBUS_PATH_CONFIG_ZONE = DBUS_PATH+"/config/zone"
 
+# Polkit actions
 _PK_ACTION = "org.fedoraproject.FirewallD%d" % DBUS_INTERFACE_VERSION
 PK_ACTION_POLICIES = _PK_ACTION+".policies"
+PK_ACTION_POLICIES_INFO = PK_ACTION_POLICIES+".info"
 PK_ACTION_CONFIG = _PK_ACTION+".config"
+PK_ACTION_CONFIG_INFO = PK_ACTION_CONFIG+".info"
 PK_ACTION_DIRECT = _PK_ACTION+".direct"
+PK_ACTION_DIRECT_INFO = PK_ACTION_DIRECT+".info"
 PK_ACTION_INFO = _PK_ACTION+".info"
-#PK_ACTION_MODIFY = _PK_ACTION+".modify"
+PK_ACTION_ALL = _PK_ACTION+".all" # implies all other actions
