@@ -119,7 +119,7 @@ class Firewall:
                 default_zone = self._firewalld_conf.get("DefaultZone")
             if self._firewalld_conf.get("MinimalMark"):
                 mark = self._firewalld_conf.get("MinimalMark")
-                if mark != None:
+                if mark is not None:
                     try:
                         self._min_mark = int(mark)
                     except Exception as msg:
@@ -127,12 +127,12 @@ class Firewall:
                                   "value %d", mark, self._min_mark)
             if self._firewalld_conf.get("CleanupOnExit"):
                 value = self._firewalld_conf.get("CleanupOnExit")
-                if value != None and value.lower() in [ "no", "false" ]:
+                if value is not None and value.lower() in [ "no", "false" ]:
                     self.cleanup_on_exit = False
 
             if self._firewalld_conf.get("Lockdown"):
                 value = self._firewalld_conf.get("Lockdown")
-                if value != None and value.lower() in [ "yes", "true" ]:
+                if value is not None and value.lower() in [ "yes", "true" ]:
                     log.debug1("Lockdown is enabled")
                     try:
                         self.policies.enable_lockdown()
@@ -142,7 +142,7 @@ class Firewall:
 
             if self._firewalld_conf.get("IPv6_rpfilter"):
                 value = self._firewalld_conf.get("IPv6_rpfilter")
-                if value != None:
+                if value is not None:
                     if value.lower() in [ "no", "false" ]:
                         self.ipv6_rpfilter_enabled = False
                     if value.lower() in [ "yes", "true" ]:
@@ -217,13 +217,8 @@ class Firewall:
                            FIREWALLD_DIRECT, msg)
             else:
                 self.direct.set_config((obj.get_all_chains(),
-                                        obj.get_all_rules()))
-                for ipv, args in obj.get_all_passthroughs().items():
-                    for arg in args:
-                        try:
-                            self.direct.passthrough(ipv, arg)
-                        except FirewallError as error:
-                            log.warning(str(error))
+                                        obj.get_all_rules(),
+                                        obj.get_all_passthroughs()))
         self.config.set_direct(copy.deepcopy(obj))
 
         # check if default_zone is a valid zone
@@ -257,7 +252,7 @@ class Firewall:
         if not os.path.isdir(path):
             return
 
-        if combine == True:
+        if combine:
             if path.startswith(ETC_FIREWALLD) and reader_type == "zone":
                 combined_zone = Zone()
                 combined_zone.name = os.path.basename(path)
@@ -341,7 +336,7 @@ class Firewall:
                 log.error("Failed to load %s file '%s':", reader_type, name)
                 log.exception()
 
-        if combine == True and combined_zone.combined == True:
+        if combine and combined_zone.combined:
             if combined_zone.name in self.zone.get_zones():
                 orig_obj = self.zone.get_zone(combined_zone.name)
                 log.debug1("  Overloading and deactivating %s '%s' ('%s/%s')",
@@ -419,6 +414,7 @@ class Firewall:
             try:
                 self.rule(ipv, [ append_delete[enable], ] + rule)
             except Exception as msg:
+                log.error("Failed to apply rules. A firewall reload might solve the issue if the firewall has been modified using ip*tables or ebtables.")
                 log.error(msg)
                 return (rules[:i], msg) # cleanup rules and error message
         return None
@@ -452,6 +448,7 @@ class Firewall:
                 self.rule(ipv, [ "-t", table,
                                  append_delete[enable], chain, ] + rule)
             except Exception as msg:
+                log.error("Failed to apply rules. A firewall reload might solve the issue if the firewall has been modified using ip*tables or ebtables.")
                 log.error(msg)
                 return (rules[:i], msg) # cleanup rules and error message
         return None
@@ -622,13 +619,13 @@ class Firewall:
     def check_port(self, port):
         range = functions.getPortRange(port)
 
-        if range == -2 or range == -1 or range == None or \
+        if range == -2 or range == -1 or range is None or \
                 (len(range) == 2 and range[0] >= range[1]):
             if range == -2:
                 log.debug1("'%s': port > 65535" % port)
             elif range == -1:
                 log.debug1("'%s': port is invalid" % port)
-            elif range == None:
+            elif range is None:
                 log.debug1("'%s': port is ambiguous" % port)
             elif len(range) == 2 and range[0] >= range[1]:
                 log.debug1("'%s': range start >= end" % port)
@@ -760,7 +757,7 @@ class Firewall:
         self._panic = False
 
     def query_panic_mode(self):
-        return (self._panic == True)
+        return self._panic
 
     # DEFAULT ZONE
 
