@@ -35,6 +35,12 @@ class FirewallConfig:
         self._fw = fw
         self.__init_vars()
 
+    def __repr__(self):
+        return '%s(%r, %r, %r, %r, %r, %r, %r, %r, %r)' % (self.__class__,
+                   self._icmptypes, self._services, self._zones,
+                   self._default_icmptypes, self._default_services, self._default_zones,
+                   self._firewalld_conf, self._policies, self._direct)
+
     def __init_vars(self):
         self._icmptypes = { }
         self._services = { }
@@ -47,21 +53,42 @@ class FirewallConfig:
         self._direct = None
 
     def cleanup(self):
-        for x in self._default_icmptypes.keys():
+        for x in list(self._default_icmptypes.keys()):
+            self._default_icmptypes[x].cleanup()
             del self._default_icmptypes[x]
-        for x in self._icmptypes.keys():
+        for x in list(self._icmptypes.keys()):
+            self._icmptypes[x].cleanup()
             del self._icmptypes[x]
 
-        for x in self._default_services.keys():
+        for x in list(self._default_services.keys()):
+            self._default_services[x].cleanup()
             del self._default_services[x]
-        for x in self._services.keys():
+        for x in list(self._services.keys()):
+            self._services[x].cleanup()
             del self._services[x]
 
-        for x in self._default_zones.keys():
+        for x in list(self._default_zones.keys()):
+            self._default_zones[x].cleanup()
             del self._default_zones[x]
-        for x in self._zones.keys():
+        for x in list(self._zones.keys()):
+            self._zones[x].cleanup()
             del self._zones[x]
 
+        if self._firewalld_conf:
+            self._firewalld_conf.cleanup()
+            del self._firewalld_conf
+            self._firewalld_conf = None
+
+        if self._policies:
+            self._policies.cleanup()
+            del self._policies
+            self._policies = None
+
+        if self._direct:
+            self._direct.cleanup()
+            del self._direct
+            self._direct = None
+
         self.__init_vars()
 
     # access check
@@ -80,6 +107,12 @@ class FirewallConfig:
     def get_firewalld_conf(self):
         return self._firewalld_conf
 
+    def update_firewalld_conf(self):
+        if not os.path.exists(LOCKDOWN_WHITELIST):
+            self._firewalld_conf.clear()
+        else:
+            self._firewalld_conf.read()
+
     # policies
 
     def set_policies(self, policies):
@@ -90,7 +123,7 @@ class FirewallConfig:
 
     def update_lockdown_whitelist(self):
         if not os.path.exists(LOCKDOWN_WHITELIST):
-            self._policies.lockdown_whitelist.clear()
+            self._policies.lockdown_whitelist.cleanup()
         else:
             self._policies.lockdown_whitelist.read()
 
@@ -104,15 +137,15 @@ class FirewallConfig:
 
     def update_direct(self):
         if not os.path.exists(FIREWALLD_DIRECT):
-            self._direct.clear()
+            self._direct.cleanup()
         else:
             self._direct.read()
 
     # icmptypes
 
     def get_icmptypes(self):
-        return sorted(set(self._icmptypes.keys() + \
-                          self._default_icmptypes.keys()))
+        return sorted(set(list(self._icmptypes.keys()) + \
+                          list(self._default_icmptypes.keys())))
 
     def add_icmptype(self, obj):
         if obj.default:
@@ -134,9 +167,15 @@ class FirewallConfig:
         return name in self._default_icmptypes
 
     def load_icmptype_defaults(self, obj):
-        if obj.name not in self._icmptypes or self._icmptypes[obj.name] != obj or \
-                obj.name not in self._default_icmptypes:
-            raise FirewallError(NO_DEFAULTS, obj.name)
+        if obj.name not in self._icmptypes:
+            raise FirewallError(NO_DEFAULTS,
+                                "'%s' not in self._icmptypes" % obj.name)
+        elif self._icmptypes[obj.name] != obj:
+            raise FirewallError(NO_DEFAULTS,
+                                "self._icmptypes[%s] != obj" % obj.name)
+        elif obj.name not in self._default_icmptypes:
+            raise FirewallError(NO_DEFAULTS,
+                            "'%s' not in self._default_icmptypes" % obj.name)
         self._remove_icmptype(obj)
         return self._default_icmptypes[obj.name]
 
@@ -163,7 +202,7 @@ class FirewallConfig:
         except:
             pass
         else:
-            raise FirewallError(NAME_CONFLICT, name)
+            raise FirewallError(NAME_CONFLICT, "new_icmptype(): '%s'" % name)
 
         x = IcmpType()
         x.check_name(name)
@@ -242,9 +281,11 @@ class FirewallConfig:
 
     def _remove_icmptype(self, obj):
         if obj.name not in self._icmptypes:
-            raise FirewallError(INVALID_ICMPTYPE, obj.name)
+            raise FirewallError(INVALID_ICMPTYPE,
+                                "'%s' not in self._icmptypes" % obj.name)
         if obj.path != ETC_FIREWALLD_ICMPTYPES:
-            raise FirewallError(INVALID_DIRECTORY, obj.path)
+            raise FirewallError(INVALID_DIRECTORY,
+                        "'%s' != '%s'" % (obj.path, ETC_FIREWALLD_ICMPTYPES))
         os.remove("%s/%s.xml" % (obj.path, obj.name))
         del self._icmptypes[obj.name]
 
@@ -255,7 +296,8 @@ class FirewallConfig:
 
     def check_builtin_icmptype(self, obj):
         if self.is_builtin_icmptype(obj):
-            raise FirewallError(BUILTIN_ICMPTYPE, obj.name)
+            raise FirewallError(BUILTIN_ICMPTYPE,
+                                "'%s' is built-in icmp type" % obj.name)
 
     def remove_icmptype(self, obj):
         self.check_builtin_icmptype(obj)
@@ -273,8 +315,8 @@ class FirewallConfig:
     # services
 
     def get_services(self):
-        return sorted(set(self._services.keys() + \
-                          self._default_services.keys()))
+        return sorted(set(list(self._services.keys()) + \
+                          list(self._default_services.keys())))
 
     def add_service(self, obj):
         if obj.default:
@@ -287,7 +329,7 @@ class FirewallConfig:
             return self._services[name]
         elif name in self._default_services:
             return self._default_services[name]
-        raise FirewallError(INVALID_SERVICE, name)
+        raise FirewallError(INVALID_SERVICE, "get_service(): '%s'" % name)
 
     def service_has_defaults(self, name):
         return (name in self._services and name in self._default_services)
@@ -296,9 +338,15 @@ class FirewallConfig:
         return name in self._default_services
 
     def load_service_defaults(self, obj):
-        if obj.name not in self._services or self._services[obj.name] != obj or \
-                obj.name not in self._default_services:
-            raise FirewallError(NO_DEFAULTS, obj.name)
+        if obj.name not in self._services:
+            raise FirewallError(NO_DEFAULTS,
+                                "'%s' not in self._services" % obj.name)
+        elif self._services[obj.name] != obj:
+            raise FirewallError(NO_DEFAULTS,
+                                "self._services[%s] != obj" % obj.name)
+        elif obj.name not in self._default_services:
+            raise FirewallError(NO_DEFAULTS,
+                            "'%s' not in self._default_services" % obj.name)
         self._remove_service(obj)
         return self._default_services[obj.name]
 
@@ -325,7 +373,7 @@ class FirewallConfig:
         except:
             pass
         else:
-            raise FirewallError(NAME_CONFLICT, name)
+            raise FirewallError(NAME_CONFLICT, "new_service(): '%s'" % name)
 
         x = Service()
         x.check_name(name)
@@ -404,9 +452,11 @@ class FirewallConfig:
 
     def _remove_service(self, obj):
         if obj.name not in self._services:
-            raise FirewallError(INVALID_SERVICE, obj.name)
+            raise FirewallError(INVALID_SERVICE,
+                                "'%s' not in self._services" % obj.name)
         if obj.path != ETC_FIREWALLD_SERVICES:
-            raise FirewallError(INVALID_DIRECTORY, obj.path)
+            raise FirewallError(INVALID_DIRECTORY,
+                        "'%s' != '%s'" % (obj.path, ETC_FIREWALLD_SERVICES))
         os.remove("%s/%s.xml" % (obj.path, obj.name))
         del self._services[obj.name]
 
@@ -417,7 +467,8 @@ class FirewallConfig:
 
     def check_builtin_service(self, obj):
         if self.is_builtin_service(obj):
-            raise FirewallError(BUILTIN_SERVICE, obj.name)
+            raise FirewallError(BUILTIN_SERVICE,
+                                "'%s' is built-in service" % obj.name)
 
     def remove_service(self, obj):
         self.check_builtin_service(obj)
@@ -435,8 +486,8 @@ class FirewallConfig:
     # zones
 
     def get_zones(self):
-        return sorted(set(self._zones.keys() + \
-                          self._default_zones.keys()))
+        return sorted(set(list(self._zones.keys()) + \
+                          list(self._default_zones.keys())))
 
     def add_zone(self, obj):
         if obj.default:
@@ -455,7 +506,7 @@ class FirewallConfig:
             return self._zones[name]
         elif name in self._default_zones:
             return self._default_zones[name]
-        raise FirewallError(INVALID_ZONE, name)
+        raise FirewallError(INVALID_ZONE, "get_zone(): %s" % name)
 
     def zone_has_defaults(self, name):
         return (name in self._zones and name in self._default_zones)
@@ -464,9 +515,15 @@ class FirewallConfig:
         return name in self._default_zones
 
     def load_zone_defaults(self, obj):
-        if obj.name not in self._zones or self._zones[obj.name] != obj or \
-                obj.name not in self._default_zones:
-            raise FirewallError(NO_DEFAULTS, obj.name)
+        if obj.name not in self._zones:
+            raise FirewallError(NO_DEFAULTS,
+                                "'%s' not in self._zones" % obj.name)
+        elif self._zones[obj.name] != obj:
+            raise FirewallError(NO_DEFAULTS,
+                                "self._zones[%s] != obj" % obj.name)
+        elif obj.name not in self._default_zones:
+            raise FirewallError(NO_DEFAULTS,
+                                "'%s' not in self._default_zones" % obj.name)
         self._remove_zone(obj)
         return self._default_zones[obj.name]
 
@@ -495,7 +552,7 @@ class FirewallConfig:
         except:
             pass
         else:
-            raise FirewallError(NAME_CONFLICT, name)
+            raise FirewallError(NAME_CONFLICT, "new_zone(): '%s'" % name)
 
         x = Zone()
         x.check_name(name)
@@ -576,10 +633,12 @@ class FirewallConfig:
 
     def _remove_zone(self, obj):
         if obj.name not in self._zones:
-            raise FirewallError(INVALID_ZONE, obj.name)
-        if obj.path != ETC_FIREWALLD_ZONES:
-            raise FirewallError(INVALID_DIRECTORY, obj.path)
-        os.remove("%s/%s.xml" % (obj.path, obj.name))
+            raise FirewallError(INVALID_ZONE,
+                                "'%s' not in self._zones" % obj.name)
+        if not obj.path.startswith(ETC_FIREWALLD_ZONES):
+            raise FirewallError(INVALID_DIRECTORY,
+                "'%s' doesn't start with '%s'" % (obj.path, ETC_FIREWALLD_ZONES))
+        os.remove("%s/%s.xml" % (ETC_FIREWALLD_ZONES, obj.name))
         del self._zones[obj.name]
 
     def is_builtin_zone(self, obj):
@@ -589,7 +648,7 @@ class FirewallConfig:
 
     def check_builtin_zone(self, obj):
         if self.is_builtin_zone(obj):
-            raise FirewallError(BUILTIN_ZONE, obj.name)
+            raise FirewallError(BUILTIN_ZONE, "'%s' is built-in zone" % obj.name)
 
     def remove_zone(self, obj):
         self.check_builtin_zone(obj)
