@@ -55,15 +55,16 @@ class FirewallDirect:
         self._chains = LastUpdatedOrderedDict()
         self._rules = LastUpdatedOrderedDict()
         self._rule_priority_positions = { }
+        self._passthroughs = LastUpdatedOrderedDict()
 
     def cleanup(self):
         self.__init_vars()
 
     def get_config(self):
-        return (self._chains, self._rules)
+        return (self._chains, self._rules, self._passthroughs)
 
     def set_config(self, config):
-        (_chains, _rules) = config
+        (_chains, _rules, _passthroughs) = config
         for table_id in _chains:
             (ipv, table) = table_id
             for chain in _chains[table_id]:
@@ -82,11 +83,23 @@ class FirewallDirect:
                     except FirewallError as error:
                         log.warning(str(error))
 
-    def _check_ipv_table(self, ipv, table):
-        ipvs = [ 'ipv4', 'ipv6', 'eb' ]
+        for ipv in _passthroughs:
+            for args in _passthroughs[ipv]:
+                if not self.query_passthrough(ipv, args):
+                    try:
+                        self.add_passthrough(ipv, args)
+                    except FirewallError as error:
+                        log.warning(str(error))
+
+    def _check_ipv(self, ipv):
+        ipvs = ['ipv4', 'ipv6', 'eb']
         if ipv not in ipvs:
             raise FirewallError(INVALID_IPV,
                                 "'%s' not in '%s'" % (ipv, ipvs))
+
+    def _check_ipv_table(self, ipv, table):
+        self._check_ipv(ipv)
+
         tables = ipXtables.BUILT_IN_CHAINS.keys() if ipv in [ 'ipv4', 'ipv6' ] \
                                          else ebtables.BUILT_IN_CHAINS.keys()
         if table not in tables:
@@ -309,10 +322,10 @@ class FirewallDirect:
         for key in self._rules:
             (ipv, table, chain) = key
             for (priority, args) in self._rules[key]:
-                r.append((ipv, table, chain, priority, args))
+                r.append((ipv, table, chain, priority, list(args)))
         return r
 
-    # DIRECT PASSTROUGH
+    # DIRECT PASSTHROUGH (untracked)
 
     def passthrough(self, ipv, args):
         try:
@@ -320,3 +333,135 @@ class FirewallDirect:
         except Exception as msg:
             log.debug2(msg)
             raise FirewallError(COMMAND_FAILED, msg)
+
+    # DIRECT PASSTHROUGH (tracked)
+
+    def __passthrough(self, enable, ipv, args):
+        self._check_ipv(ipv)
+
+        passthrough_id = (ipv, args)
+        if enable:
+            if ipv in self._passthroughs and args in self._passthroughs[ipv]:
+                raise FirewallError(ALREADY_ENABLED,
+                                    "passthrough '%s', '%s'" % (ipv, args))
+        else:
+            if not ipv in self._passthroughs or \
+               args not in self._passthroughs[ipv]:
+                raise FirewallError(NOT_ENABLED,
+                                    "passthrough '%s', '%s'" % (ipv, args))
+
+        if enable:
+            self.check_passthrough(args)
+            _args = args
+        else:
+            _args = self.reverse_passthrough(args)
+
+        try:
+            self._fw.rule(ipv, _args)
+        except Exception as msg:
+            log.debug2(msg)
+            raise FirewallError(COMMAND_FAILED, msg)
+
+        if enable:
+            if not ipv in self._passthroughs:
+                self._passthroughs[ipv] = [ ]
+            self._passthroughs[ipv].append(args)
+        else:
+            self._passthroughs[ipv].remove(args)
+            if len(self._passthroughs[ipv]) == 0:
+                del self._passthroughs[ipv]
+
+    def add_passthrough(self, ipv, args):
+        self.__passthrough(True, ipv, list(args))
+
+    def remove_passthrough(self, ipv, args):
+        self.__passthrough(False, ipv, list(args))
+
+    def query_passthrough(self, ipv, args):
+        return (ipv in self._passthroughs and \
+                list(args) in self._passthroughs[ipv])
+
+    def get_all_passthroughs(self):
+        r = [ ]
+        for ipv in self._passthroughs:
+            for args in self._passthroughs[ipv]:
+                r.append((ipv, list(args)))
+        return r
+
+    def get_passthroughs(self, ipv):
+        r = [ ]
+        if ipv in self._passthroughs:
+            for args in self._passthroughs[ipv]:
+                r.append(list(args))
+        return r
+
+    def check_passthrough(self, args):
+        """ Check if passthough rule is valid (only add, insert and new chain
+        rules are allowed) """
+
+        args = set(args)
+        not_allowed = set(["-C", "--check",           # check rule
+                           "-D", "--delete",          # delete rule
+                           "-R", "--replace",         # replace rule
+                           "-L", "--list",            # list rule
+                           "-S", "--list-rules",      # print rules
+                           "-F", "--flush",           # flush rules
+                           "-Z", "--zero",            # zero rules
+                           "-X", "--delete-chain",    # delete chain
+                           "-P", "--policy",          # policy
+                           "-E", "--rename-chain"])   # rename chain)
+        # intersection of args and not_allowed is not empty, i.e.
+        # something from args is not allowed
+        if len(args & not_allowed) > 0:
+                raise FirewallError(INVALID_PASSTHROUGH,
+                                    "arg '%s' is not allowed" %
+                                    list(args & not_allowed)[0])
+
+        # args need to contain one of -A, -I, -N
+        needed = set(["-A", "--append",
+                      "-I", "--insert",
+                      "-N", "--new-chain"])
+        # empty intersection of args and needed, i.e.
+        # none from args contains any needed command
+        if len(args & needed) == 0:
+            raise FirewallError(INVALID_PASSTHROUGH,
+                                "no '-A', '-I' or '-N' arg")
+
+    def reverse_passthrough(self, args):
+        """ Reverse valid passthough rule """
+
+        replace_args = {
+            # Append
+            "-A": "-D",
+            "--append": "--delete",
+            # Insert
+            "-I": "-D",
+            "--insert": "--delete",
+            # New chain
+            "-N": "-X",
+            "--new-chain": "--delete-chain",
+        }
+
+        ret_args = args[:]
+
+        for x in replace_args:
+            try:
+                idx = ret_args.index(x)
+            except:
+                continue
+
+            if x in [ "-I", "--insert" ]:
+                # With insert rulenum, then remove it if it is a number
+                # Opt at position idx, chain at position idx+1, [rulenum] at
+                # position idx+2
+                try:
+                    int(ret_args[idx+2])
+                except:
+                    pass
+                else:
+                    ret_args.pop(idx+2)
+
+            ret_args[idx] = replace_args[x]
+            return ret_args
+
+        raise FirewallError(INVALID_PASSTHROUGH, "no '-A', '-I' or '-N' arg")
