@@ -28,6 +28,9 @@ import syslog
 import traceback
 import fcntl
 
+PY2 = sys.version < '3'
+PY3 = sys.version >= '3'
+
 # ---------------------------------------------------------------------------
 
 # abstract class for logging targets
@@ -95,7 +98,7 @@ class _SyslogLog(_StdoutLog):
         if data.endswith("\n"):
             data = data[:len(data)-1]
         if len(data) > 0:
-            if priority == None:
+            if priority is None:
                 syslog.syslog(data)
             else:
                 syslog.syslog(priority, data)
@@ -188,7 +191,7 @@ class Logger:
     from logger import log
     log.setInfoLogLevel(log.INFO1)
     log.setDebugLogLevel(log.DEBUG1)
-    for i in xrange(1, log.INFO_MAX+1):
+    for i in range(1, log.INFO_MAX+1):
         log.setInfoLogLabel(i, "INFO%d: " % i)
     log.setFormat("%(date)s %(module)s:%(line)d [%(domain)s] %(label)s: "
                   "%(level)d %(message)s")
@@ -254,7 +257,7 @@ class Logger:
         self.setInfoLogLabel(self.WARNING, "WARNING: ")
 
         # generate info levels and infox functions
-        for _level in xrange(1, self.INFO_MAX+1):
+        for _level in range(1, self.INFO_MAX+1):
             setattr(self, "INFO%d" % _level, _level)
             self.setInfoLogLabel(_level, "")
             setattr(self, "info%d" % (_level),
@@ -263,7 +266,7 @@ class Logger:
                      self.info(x, message, *args, **kwargs))(self, _level))
 
         # generate debug levels and debugx functions
-        for _level in xrange(1, self.DEBUG_MAX+1):
+        for _level in range(1, self.DEBUG_MAX+1):
             setattr(self, "DEBUG%d" % _level, _level)
             self.setDebugLogLabel(_level, "DEBUG%d: " % _level)
             setattr(self, "debug%d" % (_level),
@@ -278,13 +281,13 @@ class Logger:
         self.setDateFormat("%d %b %Y %H:%M:%S")
         self.setInfoLogging("*", self.stderr, [ self.FATAL, self.ERROR ])
         self.setInfoLogging("*", self.stdout,
-                        [ i for i in xrange(self.WARNING, self.INFO_MAX+1) ])
+                        [ i for i in range(self.WARNING, self.INFO_MAX+1) ])
         self.setDebugLogging("*", self.stdout,
-                             [ i for i in xrange(1, self.DEBUG_MAX+1) ])
+                             [ i for i in range(1, self.DEBUG_MAX+1) ])
 
     def close(self):
         """ Close all logging targets """
-        for level in xrange(self.FATAL, self.DEBUG_MAX+1):
+        for level in range(self.FATAL, self.DEBUG_MAX+1):
             if not level in self._logging:
                 continue
             for (domain, target, _format) in self._logging[level]:
@@ -470,9 +473,9 @@ class Logger:
                                         max=self.INFO_MAX)
         else:
             if is_debug:
-                levels = [ i for i in xrange(self.DEBUG1, self.DEBUG_MAX) ]
+                levels = [ i for i in range(self.DEBUG1, self.DEBUG_MAX) ]
             else:
-                levels = [ i for i in xrange(self.FATAL, self.INFO_MAX) ]
+                levels = [ i for i in range(self.FATAL, self.INFO_MAX) ]
         return levels
 
     def _getTargets(self, target):
@@ -502,7 +505,7 @@ class Logger:
         if len(_domains) > 0:
             _domains.clear()
 
-        for level in xrange(_range[0], _range[1]):
+        for level in range(_range[0], _range[1]):
             if not level in _logging:
                 continue
             for (domain, target, _format) in _logging[level]:
@@ -601,16 +604,17 @@ class Logger:
         # function in module?
         if code.co_name in module.__dict__:
             if hasattr(module.__dict__[code.co_name], "func_code") and \
-                   module.__dict__[code.co_name].func_code  == code:
+                   module.__dict__[code.co_name].__code__  == code:
                 return None
 
         # class in module
-        for (name, obj) in module.__dict__.iteritems():
-            if isinstance(obj, types.ClassType):
+        for (name, obj) in module.__dict__.items():
+            if ((PY2 and isinstance(obj, types.ClassType)) or
+                (PY3 and isinstance(obj, type))):
                 if hasattr(obj, code.co_name):
                     value = getattr(obj, code.co_name)
                     if type(value) == types.FunctionType:
-                        if value.func_code == code:
+                        if value.__code__ == code:
                             return obj
 
         # nothing found
@@ -620,7 +624,7 @@ class Logger:
         """ Internal function to get calling class. Returns class or None. """
         for value in obj.__dict__.values():
             if type(value) == types.FunctionType:
-                if value.func_code == code:
+                if value.__code__ == code:
                     return obj
 
         for base in obj.__bases__:
@@ -819,9 +823,9 @@ log = Logger()
 if __name__ == '__main__':
     log.setInfoLogLevel(log.INFO2)
     log.setDebugLogLevel(log.DEBUG5)
-    for i in xrange(log.INFO1, log.INFO_MAX+1):
+    for i in range(log.INFO1, log.INFO_MAX+1):
         log.setInfoLogLabel(i, "INFO%d: " % i)
-    for i in xrange(log.DEBUG1, log.DEBUG_MAX+1):
+    for i in range(log.DEBUG1, log.DEBUG_MAX+1):
         log.setDebugLogLabel(i, "DEBUG%d: " % i)
 
     log.setFormat("%(date)s %(module)s:%(line)d %(label)s"
