@@ -22,18 +22,23 @@
 from firewall import functions
 from firewall.errors import *
 
+REJECT_TYPES = {
+    "ipv4": ["icmp-host-prohibited", "host-prohib", "icmp-net-unreachable", "net-unreach", "icmp-host-unreachable", "host-unreach", "icmp-port-unreachable", "port-unreach", "icmp-proto-unreachable", "proto-unreach", "icmp-net-prohibited", "net-prohib", "tcp-reset", "tcp-rst", "icmp-admin-prohibited", "admin-prohib"],
+    "ipv6": ["icmp6-adm-prohibited", "adm-prohibited", "icmp6-no-route", "no-route", "icmp6-addr-unreachable", "addr-unreach", "icmp6-port-unreachable", "port-unreach", "tcp-reset"]
+}
+
 class Rich_Source(object):
     def __init__(self, addr, invert=False):
         self.addr = addr
         self.invert = invert
 
     def __str__(self):
-        return 'source %saddress="%s"' % ("NOT " if self.invert else "",
+        return 'source %saddress="%s"' % ("not " if self.invert else "",
                                           self.addr)
 
 class Rich_Destination(Rich_Source):
     def __str__(self):
-        return 'destination %saddress="%s"' % ("NOT " if self.invert else "",
+        return 'destination %saddress="%s"' % ("not " if self.invert else "",
                                                self.addr)
 
 class Rich_Service(object):
@@ -78,10 +83,10 @@ class Rich_ForwardPort(object):
         self.protocol = protocol
         self.to_port = to_port
         self.to_address = to_address
-        # do not use None for to_port and to_address, replace it by ""
-        if self.to_port == None:
+        # replace None with "" in to_port and/or to_address
+        if self.to_port is None:
             self.to_port = ""
-        if self.to_address == None:
+        if self.to_address is None:
             self.to_address = ""
 
     def __str__(self):
@@ -124,9 +129,18 @@ class Rich_Reject(object):
         self.limit = limit
 
     def __str__(self):
-        return "reject%s%s" % (' type="%s"' % self.type if self.type != None else "",
+        return "reject%s%s" % (' type="%s"' % self.type if self.type else "",
                                " %s" % self.limit if self.limit else "")
 
+    def check(self, family):
+        if self.type:
+            if not family:
+                raise FirewallError(INVALID_RULE, "When using reject type you must specify also rule family.")
+            if family in ['ipv4', 'ipv6'] and \
+               self.type not in REJECT_TYPES[family]:
+                valid_types = ", ".join(REJECT_TYPES[family])
+                raise FirewallError(INVALID_RULE, "Wrong reject type %s.\nUse one of: %s." % (self.type, valid_types))
+
 class Rich_Drop(Rich_Accept):
     def __str__(self):
         return "drop%s" % (" %s" % self.limit if self.limit else "")
@@ -177,7 +191,7 @@ class Rich_Limit(object):
 
 class Rich_Rule(object):
     def __init__(self, family=None, rule_str=None):
-        if family != None:
+        if family is not None:
             self.family = str(family)
         else:
             self.family = None
@@ -234,21 +248,21 @@ class Rich_Rule(object):
             #print ("in_elements: ", in_elements)
             #print ("index: %s, element: %s, attribute: %s=%s" % (index, element, attr_name, attr_value))
             if attr_name:     # attribute
-                if attr_name not in ['family', 'address', 'invert', 'value', \
-                                 'port', 'protocol', 'to-port', 'to-addr', \
-                                 'name', 'prefix', 'level', 'type']:
+                if attr_name not in ['family', 'address', 'invert', 'value',
+                                     'port', 'protocol', 'to-port', 'to-addr',
+                                     'name', 'prefix', 'level', 'type']:
                     raise FirewallError(INVALID_RULE, "bad attribute '%s'" % attr_name)
             else:             # element
-                if element in ['rule', 'source', 'destination', 'protocol', \
-                             'service', 'port', 'icmp-block', 'masquerade', \
-                             'forward-port', 'log', 'audit', \
-                             'accept', 'drop', 'reject', 'limit', 'NOT', 'EOL']:
+                if element in ['rule', 'source', 'destination', 'protocol',
+                               'service', 'port', 'icmp-block', 'masquerade',
+                               'forward-port', 'log', 'audit',
+                               'accept', 'drop', 'reject', 'limit', 'not', 'NOT', 'EOL']:
                     if element == 'source' and self.source:
                         raise FirewallError(INVALID_RULE, "more than one 'source' element")
                     elif element == 'destination' and self.destination:
                         raise FirewallError(INVALID_RULE, "more than one 'destination' element")
-                    elif element in ['protocol', 'service', 'port', 'icmp-block', \
-                                  'masquerade', 'forward-port'] and self.element:
+                    elif element in ['protocol', 'service', 'port', 'icmp-block',
+                                     'masquerade', 'forward-port'] and self.element:
                         raise FirewallError(INVALID_RULE, "more than one element. There cannot be both '%s' and '%s' in one rule." % (element, self.element))
                     elif element == 'log' and self.log:
                         raise FirewallError(INVALID_RULE, "more than one 'log' element")
@@ -287,7 +301,7 @@ class Rich_Rule(object):
             elif in_element == 'source':
                 if attr_name in ['address', 'invert']:
                     attrs[attr_name] = attr_value
-                elif element == 'NOT':
+                elif element in ['not', 'NOT']:
                     attrs['invert'] = True
                 else:
                     self.source = Rich_Source(attrs.get('address'), attrs.get('invert'))
@@ -297,7 +311,7 @@ class Rich_Rule(object):
             elif in_element == 'destination':
                 if attr_name in ['address', 'invert']:
                     attrs[attr_name] = attr_value
-                elif element == 'NOT':
+                elif element in ['not', 'NOT']:
                     attrs['invert'] = True
                 else:
                     self.destination = Rich_Destination(attrs.get('address'), attrs.get('invert'))
@@ -380,7 +394,7 @@ class Rich_Rule(object):
             elif in_element == 'reject':
                 if attr_name == 'type':
                     attrs[attr_name] = attr_value
-                if element == 'limit':
+                elif element == 'limit':
                     in_elements.append('limit')
                 else:
                     self.action = Rich_Reject(attrs.get('type'), attrs.get('limit'))
@@ -399,43 +413,43 @@ class Rich_Rule(object):
         self.check()
 
     def check(self):
-        if self.family != None and self.family not in [ "ipv4", "ipv6" ]:
+        if self.family is not None and self.family not in [ "ipv4", "ipv6" ]:
             raise FirewallError(INVALID_FAMILY, self.family)
-        if self.family == None:
-            if self.source != None or self.destination != None:
+        if self.family is None:
+            if self.source is not None or self.destination is not None:
                 raise FirewallError(MISSING_FAMILY)
             if type(self.element) == Rich_ForwardPort:
                 raise FirewallError(MISSING_FAMILY)
 
-        if self.element == None:
-            if self.action == None:
+        if self.element is None:
+            if self.action is None:
                 raise FirewallError(INVALID_RULE, "no element, no action")
-            if self.source == None:
+            if self.source is None:
                 raise FirewallError(INVALID_RULE, "no element, no source")
-            if self.destination != None:
+            if self.destination is not None:
                 raise FirewallError(INVALID_RULE, "destination action")
 
         if type(self.element) not in [ Rich_IcmpBlock,
                                        Rich_ForwardPort,
                                        Rich_Masquerade ]:
-            if self.log == None and self.audit == None and \
-                    self.action == None:
+            if self.log is None and self.audit is None and \
+                    self.action is None:
                 raise FirewallError(INVALID_RULE, "no action, no log, no audit")
 
         # source
-        if self.source != None:
-            if self.family == None:
+        if self.source is not None:
+            if self.family is None:
                 raise FirewallError(INVALID_FAMILY)
-            if self.source.addr == None or \
+            if self.source.addr is None or \
                     not functions.check_address(self.family,
                                                 self.source.addr):
                 raise FirewallError(INVALID_ADDR, str(self.source.addr))
 
         # destination
-        if self.destination != None:
-            if self.family == None:
+        if self.destination is not None:
+            if self.family is None:
                 raise FirewallError(INVALID_FAMILY)
-            if self.destination.addr == None or \
+            if self.destination.addr is None or \
                     not functions.check_address(self.family,
                                                 self.destination.addr):
                 raise FirewallError(INVALID_ADDR, str(self.destination.addr))
@@ -444,7 +458,7 @@ class Rich_Rule(object):
         if type(self.element) == Rich_Service:
             # service availability needs to be checked in Firewall, here is no
             # knowledge about this, therefore only simple check
-            if self.element.name == None or len(self.element.name) < 1:
+            if self.element.name is None or len(self.element.name) < 1:
                 raise FirewallError(INVALID_SERVICE, str(self.element.name))
 
         # port
@@ -461,17 +475,20 @@ class Rich_Rule(object):
 
         # masquerade
         elif type(self.element) == Rich_Masquerade:
-            if self.destination != None:
+            if self.destination is not None:
                 raise FirewallError(INVALID_RULE, "masquerade and destination")
+            if self.action is not None:
+                raise FirewallError(INVALID_RULE, "masquerade and action")
+
 
         # icmp-block
         elif type(self.element) == Rich_IcmpBlock:
             # icmp type availability needs to be checked in Firewall, here is no
             # knowledge about this, therefore only simple check
-            if self.element.name == None or len(self.element.name) < 1:
+            if self.element.name is None or len(self.element.name) < 1:
                 raise FirewallError(INVALID_ICMPTYPE, str(self.element.name))
-            if self.action and type(self.action) == Rich_Accept:
-                raise FirewallError(INVALID_RULE, "icmpblock and accept")
+            if self.action:
+                raise FirewallError(INVALID_RULE, "icmp-block and action")
 
         # forward-port
         elif type(self.element) == Rich_ForwardPort:
@@ -488,38 +505,40 @@ class Rich_Rule(object):
                     not functions.check_single_address(self.family,
                                                        self.element.to_address):
                 raise FirewallError(INVALID_ADDR, self.element.to_address)
-            if self.family == None:
+            if self.family is None:
                 raise FirewallError(INVALID_FAMILY)
+            if self.action is not None:
+                raise FirewallError(INVALID_RULE, "forward-port and action")
 
         # other element and not empty?
-        elif self.element != None:
+        elif self.element is not None:
             raise FirewallError(INVALID_RULE, "Unknown element %s" % 
                                 type(self.element))
 
         # log
-        if self.log != None:
+        if self.log is not None:
             if self.log.level and \
                self.log.level not in [ "emerg", "alert", "crit", "error",
                                        "warning", "notice", "info", "debug" ]:
                 raise FirewallError(INVALID_LOG_LEVEL, self.log.level)
 
-            if self.log.limit != None:
+            if self.log.limit is not None:
                 self.log.limit.check()
 
         # audit
-        if self.audit != None:
+        if self.audit is not None:
             if type(self.action) not in [ Rich_Accept, Rich_Reject, Rich_Drop ]:
                 raise FirewallError(INVALID_AUDIT_TYPE, type(self.action))
 
-            if self.audit.limit != None:
+            if self.audit.limit is not None:
                 self.audit.limit.check()
 
         # action
-        if self.action != None:
+        if self.action is not None:
             if type(self.action) == Rich_Reject:
-                if self.action.type != None and len(self.action.type) < 1:
-                    raise FirewallError(INVALID_RULE, "reject type")
-            if self.action.limit != None:
+                self.action.check(self.family)
+
+            if self.action.limit is not None:
                 self.action.limit.check()
 
     def __str__(self):
@@ -538,7 +557,9 @@ class Rich_Rule(object):
             ret += " %s" % self.audit
         if self.action:
             ret += " %s" % self.action
-        return ret
+
+        return (functions.u2b(ret)) if functions.PY2 else ret
+
 
 #class Rich_RawRule(object):
 #class Rich_RuleSet(object):
