@@ -278,13 +278,13 @@ def enable_ip_forwarding(ipv):
 
 def check_port(port):
     range = getPortRange(port)
-    if range == -2 or range == -1 or range == None or \
+    if range == -2 or range == -1 or range is None or \
             (len(range) == 2 and range[0] >= range[1]):
         if range == -2:
             log.debug2("'%s': port > 65535" % port)
         elif range == -1:
             log.debug2("'%s': port is invalid" % port)
-        elif range == None:
+        elif range is None:
             log.debug2("'%s': port is ambiguous" % port)
         elif len(range) == 2 and range[0] >= range[1]:
             log.debug2("'%s': range start >= end" % port)
