@@ -44,7 +44,8 @@ class ChooserButton:
                 button.remove(child)
             hbox = Gtk.HBox()
             self.label = Gtk.Label()
-            arrow = Gtk.Arrow(Gtk.ArrowType.DOWN, Gtk.ShadowType.IN)
+            arrow = Gtk.Arrow(arrow_type=Gtk.ArrowType.DOWN,
+                              shadow_type=Gtk.ShadowType.IN)
             hbox.set_spacing(2)
             hbox.pack_start(self.label, True, True, 0)
             hbox.pack_end(arrow, False, False, 0)
@@ -174,10 +175,10 @@ class ToolChooserButton:
 ##############################################################################
 
 if __name__ == "__main__":
-    window = Gtk.Window(Gtk.WindowType.TOPLEVEL)
+    window = Gtk.Window(type=Gtk.WindowType.TOPLEVEL)
     window.connect("delete_event", Gtk.main_quit)
 
-    button = Gtk.Button("ChooserButton")
+    button = Gtk.Button(label="ChooserButton")
     chooserbutton = ChooserButton(button, "Enabled")
 
     hbox = Gtk.HBox()
@@ -185,8 +186,8 @@ if __name__ == "__main__":
     window.add(hbox)
 
     menu = Gtk.Menu()
-    for i in xrange(10):
-        item = Gtk.CheckMenuItem("level %d" % i)
+    for i in range(10):
+        item = Gtk.CheckMenuItem(label="level %d" % i)
 #        item = Gtk.MenuItem("level %d" % i)
         menu.add(item)
     menu.show_all()
